// mz_util.c -- procedures used by multiz.c, mz_preyama.c and rm2maf.c

#include "lib.h"
#include "maf.h"
#include "mz_util.h"

struct mafComp *mafNewComp(char *src, int start, int size, char strand,
  int srcSize, int len) {
	struct mafComp *a = ckalloc(sizeof(struct mafComp));

	a->src = copy_string(src);
	a->start = start;
	a->size = size;
	a->strand = strand;
	a->srcSize = srcSize;
	a->text = ckalloc((len+1)*sizeof(char));
	a->next = NULL;
	return a;
}

int mafPos2Col(struct mafComp *c, int pos, int textSize) {
	int col, p;

	if (pos < c->start || pos >= c->start + c->size)
		fatalf("mafPos2Col: %d not in %d-%d",
		  pos, c->start, c->start + c->size - 1);
	for (col = 0, p = c->start - 1; col < textSize; ++col)
		if (c->text[col] != '-' && ++p == pos)
			break;
	if (p != pos)
		fatal("mafPos2Col: cannot happen");
	return col;
}

// Create an alignment equal to a segment of a given alignment; don't set score
struct mafAli *mafSlice(struct mafAli *a, int start_col, int beyond_col) {
	int i, len = beyond_col - start_col;
	struct mafAli *new = ckalloc(sizeof(struct mafAli));
	struct mafComp *ac, *nc, *prev = NULL;

	new->textSize = len;
	new->next = NULL;
	new->components = NULL;
	for (ac = a->components; ac != NULL; ac = ac->next) {
		nc = ckalloc(sizeof(struct mafComp));
		nc->next = NULL;
		if (prev == NULL)
			new->components = nc;
		else
			prev->next = nc;
		prev = nc;
		nc->src = copy_string(ac->src);
		nc->srcSize = ac->srcSize;
		nc->strand = ac->strand;
		nc->start = ac->start;
		for (i = 0; i < start_col; ++i)
			if (ac->text[i] != '-')
				++(nc->start);
		nc->text = ckalloc((len+1)*sizeof(char));
		for (nc->size = i = 0; i < len; ++i)
			if ((nc->text[i] = ac->text[i+start_col]) != '-')
				++(nc->size);
		nc->text[len] = '\0';
	}
	return new;
}

// flush output-list entries that start before a specified position
struct mafAli *mafFlush(struct mafAli *output_list, int pos) {
	struct mafAli *atmp;

	while (output_list != NULL && output_list->components->start <= pos) {
		mafWrite(stdout, output_list);
		atmp = output_list;
		output_list = output_list->next;
		mafAliFree(&atmp);
	}
	return output_list;
}

// insert an alignment into the output list, sorting by components->start
struct mafAli *mafInsert(struct mafAli *output_list, struct mafAli *a) {
	struct mafAli *m, *next;
	int beg1;

	if (a == NULL)
		return output_list;

	beg1 = a->components->start;
//fprintf(stderr, "mafInsert %d\n", beg1);
	if (output_list == NULL || output_list->components->start >= beg1) {
		// put a at the start of the output list
		a->next = output_list;
		return a;
	}
	for (m = output_list; (next = m->next) != NULL &&
	  next->components->start < beg1;  m = next)
		;
	// splice a between m and m->next
	a->next = next;
	m->next = a;
	return output_list;
}

/* --------- start of code to detect unused segments of alignments ---------- */

struct node *nodeNew(struct mafAli *maf) {
	struct node *new = ckalloc(sizeof(struct node));
	int sz = maf->textSize;

	new->maf = maf;
	new->used = ckalloc(sz*sizeof(char));
	while (--sz >= 0)
		new->used[sz] = 0;
	new->next = NULL;
	return new;
}

void nodeFree(struct node **n) {
	free((*n)->used);
	if ((*n)->maf != NULL)
		mafAliFree(&((*n)->maf));
	*n = NULL;
}

// Mark node's used array according to positions in maf's first component.
void nodeMark(struct node *n, int beg, int end) {
	struct mafAli *a = n->maf;
	struct mafComp *c = a->components;
	int i, sz, begc, endc;

	sz = a->textSize;
	begc = mafPos2Col(c, beg, sz);
	endc = mafPos2Col(c, end, sz);
	for (i = begc; i <= endc; ++i)
		n->used[i] = 1;
}

/* ----------- end of code to detect unused segments of alignments ---------- */
